﻿if not globalfunctions then
  globalfunctions = true;

  -- Round a number to decimals
  function TG_TS_Round(num, idp)
    if not num then return end
    if idp and idp>0 then
      local mult = 10^idp;
      return math.floor(num * mult + 0.5) / mult;
    end
    return math.floor(num + 0.5);
  end

  -- Convert GetProfessions into an table
  function TG_TS_GetProfessions()
    local skills =  {};
    local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions();
    tinsert(skills, prof1);
    tinsert(skills, prof2);
    tinsert(skills, archaeology);
    tinsert(skills, fishing);
    tinsert(skills, cooking);
    tinsert(skills, firstAid);
    return skills;
  end

  -- CURRENCY Completer
  function TG_TS_CURRENCY()
    local id, needAmount = TourGuide.tags[TourGuide.current]:match("|CURRENCY|(%d+)%s?(%d*)|");
    needAmount = tonumber(needAmount) or 1;
    local name, amount, texture, unknown, weeklyMax, totalMax = GetCurrencyInfo(id)
    TourGuide.statusframe.extend.text:SetText(string.format("%i of %i %s", amount, needAmount, name));
    if needAmount <= amount then
      TourGuide:SetTurnedIn(TourGuide.current, true);
      return true;
    end
    return false;
  end

  -- DONTHAVESKILL Completer
  function TG_TS_DONTHAVESKILL()
    local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|DONTHAVESKILL|([%a:%s]+)%s?(%d+)|");
    SkillName = strtrim(SkillName);
    SkillLevel = tonumber(SkillLevel) or 1;

    -- Loop all curent skill and check if the skill match
    for _, sID in ipairs(TG_TS_GetProfessions()) do
      if sID then
        local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
        if SkillName == sName and SkillLevel < tonumber(sLevel) then
          TourGuide:SetTurnedIn(TourGuide.current, true);
          return true;
        end
      end
    end
    return false;
  end

  -- HAVESKILL Completer
  function TG_TS_HAVESKILL()
    local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|HAVESKILL|([%a:%s]+)%s?(%d+)|");
    SkillName = strtrim(SkillName);
    SkillLevel = tonumber(SkillLevel) or 1;

    -- Loop all curent skill and check if the skill match
    for _, sID in ipairs(TG_TS_GetProfessions()) do
      if sID then
        local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
        if SkillName == sName and SkillLevel >= tonumber(sLevel) then
          TourGuide:SetTurnedIn(TourGuide.current, true);
          return true;
        end
      end
    end
    return false;
  end

  -- LEARNITEM Completer
  function TG_TS_LEARNITEM(msg)
    -- Get item name to learn
    local ItemName = TourGuide.tags[TourGuide.current]:match("|LEARNITEM|([%a:%s]+)|");
    -- Check if there is a message containing the item name
    if msg then
      if msg:match("You have learned how to create a new item: ([%a:%s]+).") then
        -- Get item nave of just learned
        local LearnedItemName = msg:match("You have learned how to create a new item: ([%a:%s]+).");

        -- Compleate if match
        if (strtrim(ItemName) == strtrim(LearnedItemName)) then
          TourGuide:SetTurnedIn(TourGuide.current, true);
          return true;
        end
      end
    -- Check for allready known items
    else
      local ProfList = {"Alchemy", "Archaeology", "Blacksmithing", "Cooking", "Enchanting", "Engineering", "First Aid", "Inscription", "Jewelcrafting", "Leatherworking", "Mining", "Tailoring", "UnknownRecipe"}
      for _, Pvalue in pairs(ProfList) do
        if TradeSkillDB[Pvalue] then
          for _, RecipeName in pairs(TradeSkillDB[Pvalue]) do
            if RecipeName == strtrim(ItemName) then
              TourGuide:SetTurnedIn(TourGuide.current, true);
              return true;
            end
          end
        end
      end
    end
    return false;
  end

  -- LOOT Completer
  function TG_TS_LOOT()
    local lootitem, lootqty = TourGuide.tags[TourGuide.current]:match("|LOOT|(%d+)%s?(%d*)|");
    lootqty = tonumber(lootqty) or 1;
    local itemName = GetItemInfo(lootitem) or "None";
    TourGuide.statusframe.extend.text:SetText(string.format("%i of %i %s", GetItemCount(lootitem), lootqty, itemName));
    if (GetItemCount(lootitem) >= lootqty) then
      TourGuide:SetTurnedIn(TourGuide.current, true);
      return true;
    end
    return false;
  end

  -- LVL Completer
  function TG_TS_LVL(level)
    lvl = tonumber(TourGuide.tags[TourGuide.current]:match("|LVL|(%d+)|"))
    if level then
      if lvl and lvl <= tonumber(level) then
        TourGuide:SetTurnedIn(TourGuide.current, true);
        return true;
      end
    else
      if lvl and lvl <= UnitLevel("Player") then
        TourGuide:SetTurnedIn(TourGuide.current, true);
        return true;
      end
    end
    return false;
  end

  -- LVLTOSKILL Completer
  function TG_TS_LVLTOSKILL()
    local SkillName, SkillLevel = TourGuide.tags[TourGuide.current]:match("|LVLTOSKILL|([%a:%s]+)%s?(%d+)|");
    SkillName = strtrim(SkillName);
    SkillLevel = tonumber(SkillLevel) or 1;
    for _, sID in ipairs(TG_TS_GetProfessions()) do
      if sID then
        local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
        if SkillName == sName then
          TourGuide.statusframe.extend.text:SetText(string.format("%s Skill %i of %i", SkillName, tonumber(sLevel), SkillLevel));
          if SkillLevel <= tonumber(sLevel) then
            TourGuide:SetTurnedIn(TourGuide.current, true);
            return true;
          end
        end
      end
    end
    return false;
  end

  -- MAP Level Completer
  function TG_TS_MAP()
    local info = TourGuide.tags[TourGuide.current]:match("|MAP|([%a%d:%s',_]+)|");
    if info then
      local _,_,MinLVL,MaxLVL = strsplit(",", info)
      -- Check if player are lower level than the map requires and complete if so, since we don't need to show it
      if MinLVL and UnitLevel("Player") < tonumber(strtrim(MinLVL)) then
        TourGuide:SetTurnedIn(TourGuide.current, true);
        return true
      end
      -- Check if player are higher level than the map requires and complete if so, since we don't need to show it
      if MaxLVL and UnitLevel("Player") > tonumber(strtrim(MaxLVL)) then
        TourGuide:SetTurnedIn(TourGuide.current, true);
        return true
      end
    end
    return false;
  end

  -- SKILL Completer
  function TG_TS_SKILL()
    local info = { strsplit(",", TourGuide.tags[TourGuide.current]:match("|SKILL|(.+)|")) }
    local numDone = 0
    for i = 1, #info, 1 do
      if strlen(strtrim(info[i])) > 0 then
        local SkillName, SkillLevel = info[i]:match("([%a:%s]+)%s?(%d+)");
        SkillName = strtrim(SkillName);
        SkillLevel = tonumber(SkillLevel) or 1;

        -- Loop all curent skill and check if the skill match
        for _, sID in ipairs(TG_TS_GetProfessions()) do
          if sID then
            local sName, _, sLevel, _, _, _, _, _ = GetProfessionInfo(sID)
            if SkillName == sName and SkillLevel <= tonumber(sLevel) then
              numDone = numDone + 1
            end
          end
        end
      -- ensure that it will complete even that there is a , to mutch
      else
        numDone = numDone + 1
      end
    end
    if numDone >= #info then
      TourGuide:SetTurnedIn(TourGuide.current, true)
      return true
    else
      return false
    end
  end

  -- TRAINSKILL Completer
  function TG_TS_TRAINSKILL(msg)
    local SkillName, MaxSkillLevel = TourGuide.tags[TourGuide.current]:match("|TRAINSKILL|([%a:%s]+)%s?(%d+)|");
    SkillName = strtrim(SkillName);
    MaxSkillLevel = tonumber(MaxSkillLevel) or 75;
    if msg then
      local abilityname = strtrim(msg:match("You .+%[(.+)%]."));
      -- Get Skill info to check
      SkillName = strtrim(SkillName);
      -- If Ability name and skillname match
      if abilityname == SkillName then
        TourGuide:SetTurnedIn(TourGuide.current, true);
      end
    else
      for _, sID in ipairs(TG_TS_GetProfessions()) do
        if sID then
          local sName, _, _, maxSLevel, _, _, _, _ = GetProfessionInfo(sID)
          if SkillName == strtrim(sName) and MaxSkillLevel <= tonumber(maxSLevel) then
            TourGuide:SetTurnedIn(TourGuide.current, true);
            return true;
          end
        end
      end
    end
    return false;
  end
  
  -- Update Skill List
  function TG_TS_UPDATESKILLLIST(ProfessionName)
    TradeSkillDB[ProfessionName] = {}
    local expanded, expIndex = {}, 1
    -- Expand all headers
    for index = GetNumTradeSkills(), 1, -1 do
      local skillName, skillType, numAvailable,  isExpanded, serviceType, numSkillUps  = GetTradeSkillInfo(index)
      if skillType == "header" then
        if not isExpanded then
          ExpandTradeSkillSubClass(index)
          expanded[expIndex] = skillName
          expIndex = expIndex + 1;
        end
      end
    end
    -- Collect Data and Collaps headers that was not expanded.
    for index = GetNumTradeSkills(), 1, -1 do
      local skillName, skillType, numAvailable,  isExpanded, serviceType, numSkillUps  = GetTradeSkillInfo(index)
      if skillType ~= "header" then
        tinsert(TradeSkillDB[ProfessionName], skillName)
        -- Remove Recipe from UnknownRecipe list if it's listed in there
        if #TradeSkillDB["UnknownRecipe"] > 0 then
          for URIndex = #TradeSkillDB["UnknownRecipe"], 1, -1 do
            if TradeSkillDB["UnknownRecipe"][URIndex] == skillName then
              tremove(TradeSkillDB["UnknownRecipe"], URIndex)
            end
          end
        end
      else
        for i = 1, #expanded, 1 do
          if skillName == expanded[i] then
            CollapseTradeSkillSubClass(index)
          end
        end
      end
    end
  end
end